import { Readable } from 'stream';
import { BATCH_SIZE, MAINTENANCE_MODE_AUTO } from './constants';
import { AlreadyFrozenError, AlreadyUnfrozenError } from './frozen-errors';

async function startProcessSecrets({
  pool,
  maintenanceMode
}) {
  if (maintenanceMode === MAINTENANCE_MODE_AUTO) {
    try {
      await pool.freeze();
    } catch (error) {
      if (!AlreadyFrozenError.is(error)) {
        throw error;
      }
    }
  }
}

async function endProcessSecrets({
  pool,
  maintenanceMode
}) {
  if (maintenanceMode === MAINTENANCE_MODE_AUTO) {
    try {
      await pool.unfreeze();
    } catch (error) {
      if (!AlreadyUnfrozenError.is(error)) {
        throw error;
      }
    }
  }
}

async function* generator(context) {
  const {
    pool
  } = context;
  await pool.waitConnect();
  await startProcessSecrets(context);

  while (true) {
    const {
      secrets,
      idx
    } = await pool.loadSecrets({
      idx: context.idx,
      limit: BATCH_SIZE
    });

    for (const secret of secrets) {
      const chunk = Buffer.from(JSON.stringify(secret) + '\n', 'utf8');
      yield chunk;
      context.idx = idx;
    }

    if (secrets.length === 0) {
      await endProcessSecrets(context);
      return;
    }
  }
}

const exportSecretsStream = (pool, {
  idx = null,
  maintenanceMode = MAINTENANCE_MODE_AUTO
} = {}) => {
  if (pool.loadSecrets === undefined) throw new Error('loadSecrets is not defined for this adapter');
  const context = {
    pool,
    idx,
    maintenanceMode
  };
  const stream = Readable.from(generator(context));
  Object.defineProperty(stream, 'idx', {
    get() {
      return context.idx;
    }

  });
  return stream;
};

export default exportSecretsStream;
//# sourceMappingURL=export-secrets.js.map